﻿CompilerIf Defined(INCLUDE_SCROLLER, #PB_Constant)=0
#INCLUDE_SCROLLER=1
;/////////////////////////////////////////////////////////////////////////////////
;***Scroller*** - Version 1.0.
;
;©nxSoftware 2013.
;=================
;   Created with Purebasic 5.21 LTS for Windows.
;
;   Platforms:  Windows XP and above.
;               No promises with earlier versions of Windows!
;
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;-General notes.
;
;   i)  This utility is wrapped within a module to protect internal functions and variables etc.
;/////////////////////////////////////////////////////////////////////////////////


;-Module declaration.
;-===================

DeclareModule Scroller

;/////////////////////////////////////////////////////////////////////////////////
;-PUBLIC CONSTANTS.

  ;Use the following constant for the 'documentWidth' and/or 'documentHeight' parameter (when creating a Scroller) to remove the appropriate scrollbar,
  ;in effect having the appropriate inner dimensions auto size to the control's client area.
    #Scroller_AUTOSIZE = -1
    
  ;Flags constants.
  ;Use a combination of the following for the optional 'flags' when creating a new Scroller. 
    Enumeration
      #Scroller_BORDERLESS = #PB_Container_BorderLess
      #Scroller_FLAT = #PB_Container_Flat
      #Scroller_RAISED = #PB_Container_Raised
      #Scroller_SINGLE = #PB_Container_Single
      #Scroller_DOUBLE = #PB_Container_Double 
      #Scroller_ADDKEYBOARDCONTROL = #Scroller_DOUBLE << 1        ;The cursor keys will cause the control to scroll. 
      #Scroller_NOERASEBKGND = #Scroller_ADDKEYBOARDCONTROL << 1  ;Supress default Window's processing of #WM_ERASEBKGND messages.
    EndEnumeration

  ;Event constants.
    Enumeration 1
      #Scroller_ONSCROLL    ;Without this handler the scrollbars will not function (unless auto scroll is set).
                            ;Return value is ignored at this time.
                            ;Use the \SetScrollX() and \SetScrollY() methods in this handler to set the actual scroll position.
      #Scroller_ONPAINT     ;Return zero for default Window's processing.
    EndEnumeration

  ;Scroll codes. Passed in the 'scrollCode' field of the Scroller_EVENT structure to #Scroller_ONSCROLL messages.
    Enumeration 1
      #Scroller_LINELEFTUP
      #Scroller_LINERIGHTDOWN
      #Scroller_PAGELEFTUP
      #Scroller_PAGERIGHTDOWN
      #Scroller_THUMBTRACK       ;User is dragging the scrollbox.
      #Scroller_THUMBPOSITION    ;User has finished dragging the scrollbox.
      #Scroller_TOP              ;Scrolls to the top-left.
      #Scroller_BOTTOM           ;Scrolls to the bottom right.
      #Scroller_MOUSEWHEEL
    EndEnumeration
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-PUBLIC STRUCTURES.

  ;A pointer to an instance of the following structure is passed to all event handlers.
  ;Indeed, all event handlers take only this one parameter.
    Structure Scroller_EVENT
      ;Fields for use by all events.
        eventCode.i               ;One of the above event constants; #Scroller_ONSCROLL or #Scroller_ONPAINT.
        pageHeight.i            ;The height of the client area which is also the height of the vertical scroll page size.
        pageWidth.i             ;The width of the client area which is also the width of the horizontal scroll page size.
        documentHeight.i        ;The height of the scrollable area as set by the client app when creating the scroller.
                                  ;In the case that #Scroller_AUTOSIZE was used, this field will contain a copy of the pageHeight field.
        documentWidth.i         ;The width of the scrollable area as set by the client app when creating the scroller.
                                  ;In the case that #Scroller_AUTOSIZE was used, this field will contain a copy of the pageWidth field.
      ;Fields for use by the scrolling events.
        blnHorizontalScrollBar.i  ;True if the scroll message relates to horizontal scrolling.
        scrollCode.i              ;One of the above scroll codes.
        scrollTrackPos.i          ;For use only with the #Scroller_THUMBTRACK and #Scroller_THUMBPOSITION scroll codes.
        scrollMouseWheelDelta.i   ;For use only if the scrollCode notifies the use of the mouse wheel.
    EndStructure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-PUBLIC INTERFACES.

  ;The main Scroller class. A pointer to such an object is returned from the Scroller_New() function.
    Interface Scroller
      Destroy()
      ;Control methods.
        Disable(state)
        GetHeight.i()                             ;Returns the height of the whole control (the container).
        GetID.i()                       ;Returns the gadget# of the underlying container gadget.
        GetUserData.i()
        GetWidth.i()                              ;Returns the width of the whole control (the container).
        GetX.i()
        GetY.i()
        Hide(state)
        Resize(x, y, width, height)     ;Can use #PB_Ignore for any parameter.
        SetFocus()
        SetUserData(useData)
      ;Client area methods.
        GetPageHeight.i()             ;Returns the actual drawing height (excludes any scrollbar which might be visible).
        GetPageWidth.i()              ;Returns the actual drawing width (excludes any scrollbar which might be visible).
      ;Scrolling methods.
        GetDocumentHeight.i()         ;Returns the client height if documentHeight auto sizes (#Scroller_AUTOSIZE)
        GetDocumentWidth.i()          ;Returns the client height if documentWidth auto sizes (#Scroller_AUTOSIZE)
        GetScrollPositions(*scrollX.INTEGER, *scrollY.INTEGER)
                                        ;This is very fast as the values are buffered.
        ResizedocumentHeight(documentHeight=#Scroller_AUTOSIZE)
        ResizedocumentWidth(documentWidth=#Scroller_AUTOSIZE)
        SetAutoScroll(scrollStep=0)     ;Set scrollStep to zero to cancel autoscroll.
                                        ;With auto scroll set, the library will automatically calculate scroll positions
                                        ;in response to user interactions with scrollbars etc. Consequently, no scroll events are generated.
                                        ;Note that auto scrolling utilises live scrolling when the user drags the scroll box.
        SetScrollX(scrollX, blnInvalidateControl=#True)
                                        ;Use in particular in response to #Scroller_ONSCROLL events.
        SetScrollY(scrollY, blnInvalidateControl=#True)
                                        ;Use in particular in response to #Scroller_ONSCROLL events.
      ;Miscellaneous.
        SetEventHandler.i(ptrHandler)   ;ptrHandler = 0 (to remove a handler) or a pointer to a suitable event handler.
                                        ;Returns the address of the previous handler.

    EndInterface
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-SCROLLER CREATION FUNCTION.
  Declare.i NewScroller(x, y, width, height, documentWidth=#Scroller_AUTOSIZE, documentHeight=#Scroller_AUTOSIZE, flags=#Scroller_BORDERLESS)
;/////////////////////////////////////////////////////////////////////////////////

EndDeclareModule


;-Module implementation.
;-======================

Module scroller

EnableExplicit


;/////////////////////////////////////////////////////////////////////////////////
;-PROTOTYPES

  Prototype.i Scroller_ProtoEventHandler(this.Scroller, *scrollerEvent.Scroller_EVENT)
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-CONSTANTS and STRUCTURES (private)

  #WHEEL_DELTA = 120
  #Scroller_WINPROP$ = "_scrollerProp"

  ;Globals. The following will be useful in determining whether scrollbars will be shown and are used to reset the page sizes
  ;following a resize etc.
    Structure Globals
      SM_CYHSCROLL.i        ;The height of a horizontal scrollbar.
      SM_CXVSCROLL.i        ;The width of a vertical scrollbar.
    EndStructure

  ;Scroller class template.
    Structure _Scroller
      *vTable
      ;General control fields.
        flags.i
        blnSupressERASEBKGND.i
      ;Container gadget.
        conID.i
        conhWnd.i
        oldWinProc.i
      ;Client area.
        pageHeight.i      ;For convenience. Will also be the vertical page size (scrollbar).
        pageWidth.i       ;For convenience. Will also be the horizontal page size (scrollbar).
      ;Scrolling fields.
        autoScrollStep.i ;If zero then auto scroll is turned off (default).
        documentHeight.i
        documentWidth.i
        scrollX.i           ;For convenience.
        scrollY.i           ;For convenience.
      ;Event handlers.
        eventProc.Scroller_ProtoEventHandler
    EndStructure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-GLOBALS
  Global globals.Globals
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-DECLARES
  Declare Scroller_SetPageSizes(*this._Scroller)
  Declare.i Scroller_WndProc(hWnd, uMsg, wParam, lParam)
  Declare Method_GetScrollPositions(*this._Scroller, *scrollX.INTEGER, *scrollY.INTEGER)
  Declare Method_SetScrollX(*this._Scroller, scrollX, blnInvalidateControl=#True)
  Declare Method_SetScrollY(*this._Scroller, scrollY, blnInvalidateControl=#True)
;/////////////////////////////////////////////////////////////////////////////////


;-CLASS CREATION FUNCTION
;========================

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NewScroller(x, y, width, height, documentWidth=#Scroller_AUTOSIZE, documentHeight=#Scroller_AUTOSIZE, flags=#Scroller_BORDERLESS)
  Protected *this._scroller
  ;Set globals if not already set.
    If globals\SM_CYHSCROLL = 0
      globals\SM_CYHSCROLL = GetSystemMetrics_(#SM_CYHSCROLL)
      globals\SM_CXVSCROLL = GetSystemMetrics_(#SM_CXVSCROLL)
    EndIf
  ;Create a new instance of a Scroller class.
    *this = AllocateMemory(SizeOf(_Scroller))
  If *this
    ;Safeguard against crazy values for documentWidth and documentHeight.
      If documentWidth < 0
        documentWidth = #Scroller_AUTOSIZE
      EndIf
      If documentHeight < 0
        documentHeight = #Scroller_AUTOSIZE
      EndIf
    With *this
      \vTable = ?Scroller_VT
      \flags = flags
      \documentHeight = documentHeight
      \documentWidth = documentWidth
    EndWith
    *this\conID = ContainerGadget(#PB_Any, x, y, width, height, flags&$7)
    If *this\conID
      If flags&#Scroller_NOERASEBKGND
        *this\blnSupressERASEBKGND = #True
      EndIf
      *this\conhWnd = GadgetID(*this\conID)
      ;Set a window property to point to the *this structure.
        SetProp_(*this\conhWnd, #Scroller_WINPROP$, *this)
      ;Subclass the container.
        *this\oldWinProc = SetWindowLongPtr_(*this\conhWnd, #GWL_WNDPROC, @Scroller_WndProc())
      ;Set the page sizes and scroll-ranges.
        Scroller_SetPageSizes(*this)
      CloseGadgetList()
    Else
      FreeMemory(*this)
      *this = 0
    EndIf
  EndIf
  ProcedureReturn *this
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-FUNCTIONS
;==========

;/////////////////////////////////////////////////////////////////////////////////
;All events are despatched through the following function.
;The *event parameter should point to an Scroller_EVENT structure in which only the event field needs initialising.
Procedure.i Scroller_DespatchEvent(*this._Scroller, *event.Scroller_EVENT)
  Protected result, sinf.SCROLLINFO, maxX, maxY
  ;Set the common fields of the event structure.
    With *event
      \pageHeight = *this\pageHeight
      \pageWidth = *this\pageWidth
      \documentHeight = *this\documentHeight
      If \documentHeight = #Scroller_AUTOSIZE
        \documentHeight = *this\pageHeight
      EndIf
      \documentWidth = *this\documentWidth
      If \documentWidth = #Scroller_AUTOSIZE
        \documentWidth = *this\pageWidth
      EndIf
    EndWith
  Select *event\eventCode
    Case #Scroller_ONSCROLL
      ;Need to fill in the trackpos field if appropriate.
        If *event\scrollCode = #Scroller_THUMBTRACK Or *event\scrollCode = #Scroller_THUMBPOSITION
          sinf\cbSize = SizeOf(SCROLLINFO)
          sinf\fMask = #SIF_TRACKPOS
          If *event\blnHorizontalScrollBar
            GetScrollInfo_(*this\conhWnd, #SB_HORZ, sinf)
          Else
            GetScrollInfo_(*this\conhWnd, #SB_VERT, sinf)
          EndIf
          *event\scrollTrackPos = sinf\nTrackPos
        EndIf
      If *this\autoScrollStep > 0 ;Auto scroll is set.
        Select *event\scrollCode
          Case #Scroller_LINELEFTUP
            If *event\blnHorizontalScrollBar
              Method_SetScrollX(*this, *this\scrollX-*this\autoScrollStep)
            Else
              Method_SetScrollY(*this, *this\scrollY-*this\autoScrollStep)
            EndIf
          Case #Scroller_LINERIGHTDOWN
            If *event\blnHorizontalScrollBar
              Method_SetScrollX(*this, *this\scrollX+*this\autoScrollStep)
            Else
              Method_SetScrollY(*this, *this\scrollY+*this\autoScrollStep)
            EndIf
          Case #Scroller_PAGELEFTUP
            If *event\blnHorizontalScrollBar
              Method_SetScrollX(*this, *this\scrollX-*event\pageWidth)
            Else
              Method_SetScrollY(*this, *this\scrollY-*event\pageHeight)
            EndIf
          Case #Scroller_PAGERIGHTDOWN
            If *event\blnHorizontalScrollBar
              Method_SetScrollX(*this, *this\scrollX+*event\pageWidth)
            Else
              Method_SetScrollY(*this, *this\scrollY+*event\pageHeight)
            EndIf
          Case #Scroller_THUMBTRACK
            If *event\blnHorizontalScrollBar
              Method_SetScrollX(*this, *event\scrollTrackPos)
            Else
              Method_SetScrollY(*this, *event\scrollTrackPos)
            EndIf
          Case #Scroller_TOP
            Method_SetScrollX(*this, 0)
            Method_SetScrollY(*this, 0)
          Case #Scroller_BOTTOM
            ;Determine the max horizontal and vertical scrolling positions.
            ;These are easy to calculate. E.g. the max vertical scroll position = height of scrollable area - vertical page size.
              maxX = *event\documentWidth - *event\pageWidth
              maxY = *event\documentHeight - *event\pageHeight
            Method_SetScrollX(*this, maxX)
            Method_SetScrollY(*this, maxY)
          Case #Scroller_MOUSEWHEEL
            Method_SetScrollY(*this, *this\scrollY - *event\scrollMouseWheelDelta**this\autoScrollStep)
        EndSelect
      Else
        ;Invoke the handler.
          If *this\eventProc
            result = *this\eventProc(*this, *event)
          EndIf
      EndIf
    Default
      ;Invoke the handler.
        If *this\eventProc
          result = *this\eventProc(*this, *event)
        EndIf
  EndSelect
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;This function calculates and resets the required page sizes. It also sets the scroll-ranges.
;This function is called when the control is first created and whenever the client app resizes either the control or the scrollable area.
;Note that Windows will automatically adjust the scrollbox positions if a resizing renders the current position invalid. This can happen if the
;control is enlarged etc.
Procedure Scroller_SetPageSizes(*this._Scroller)
  Protected sinf.SCROLLINFO, rc.RECT, rcWin.RECT, blnVscrollRequired
  ;Calculate the control dimensions without scrollbars and excluding borders.
    GetClientRect_(*this\conhWnd, rc)
    GetWindowRect_(*this\conhWnd, rcWin)
    ;Add on the width of the vertical scrollbar if required.
      If rcWin\right - rcWin\left - rc\right >= globals\SM_CXVSCROLL
        rc\right + globals\SM_CXVSCROLL
      EndIf
    ;Add on the height of the horizontal scrollbar if required.
      If rcWin\bottom - rcWin\top - rc\bottom >= globals\SM_CYHSCROLL
        rc\bottom + globals\SM_CYHSCROLL
      EndIf
  ;Our initial page sizes equate to the dimensions of the client area.
    *this\pageHeight = rc\bottom
    *this\pageWidth = rc\right
  If *this\pageHeight And *this\pageWidth
    ;In the case that a vertical scrollbar is required, we have to adjust the horizontal page size.
      If *this\documentHeight > *this\pageHeight
        *this\pageWidth - globals\SM_CXVSCROLL
        blnVscrollRequired = #True
      EndIf    
    ;In the case that a horizontal scrollbar is required, we have to adjust the vertical page size.
      If *this\documentWidth > *this\pageWidth
        *this\pageHeight - globals\SM_CYHSCROLL
        ;In the case that a vertical scrollbar was not required, we may find that the reduction in the vertical page size now means
        ;that a vertical scrollbar is required.
          If blnVscrollRequired = #False And *this\documentHeight > *this\pageHeight
            *this\pageWidth - globals\SM_CXVSCROLL
          EndIf    
      EndIf    
  EndIf
  ;Now set new page sizes and scroll ranges.
    sinf\fMask = #SIF_PAGE | #SIF_RANGE
  ;Set the vertical page size.
    With sinf
      \nPage = *this\pageHeight
      \nMin = 0
      \nMax = *this\documentHeight - 1
    EndWith
    SetScrollInfo_(*this\conhWnd, #SB_VERT, sinf, #True)
  ;Set the horizontal page size.
    With sinf
      \nPage = *this\pageWidth
      \nMin = 0
      \nMax = *this\documentWidth - 1
    EndWith
    SetScrollInfo_(*this\conhWnd, #SB_HORZ, sinf, #True)
  ;Record the current scroll positions in case they have changed.
    sinf\fMask = #SIF_POS
    GetScrollInfo_(*this\conhWnd, #SB_HORZ, sinf)
    *this\scrollX = sinf\nPos
    GetScrollInfo_(*this\conhWnd, #SB_VERT, sinf)
    *this\scrollY = sinf\nPos
  InvalidateRect_(*this\conhWnd, 0, 1)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The Scroller window procedure.
Procedure.i Scroller_WndProc(hWnd, uMsg, wParam, lParam)
  Protected result, *this._Scroller, event.Scroller_EVENT, scrollCode, oldProc, zDelta.w
  *this = GetProp_(hWnd, #Scroller_WINPROP$)
  If *this
    oldProc = *this\oldWinProc
  Else
    ProcedureReturn DefWindowProc_(hWnd, uMsg, wParam, lParam)
  EndIf
  Select uMsg
    Case #WM_LBUTTONDOWN
      SetFocus_(hWnd)
      result = CallWindowProc_(oldProc, hWnd, uMsg, wParam, lParam)
    Case #WM_PAINT
      event\eventCode = #Scroller_ONPAINT
      result = Scroller_DespatchEvent(*this, event)
      If result = 0
        result = CallWindowProc_(oldProc, hWnd, uMsg, wParam, lParam)
      EndIf
    Case #WM_ERASEBKGND
      If *this\blnSupressERASEBKGND = 0
        result = CallWindowProc_(oldProc, hWnd, uMsg, wParam, lParam)
      Else
        result = 1  ;This value means that Windows will mark the client area as having been erased. This will reduce flicker on older systems.
      EndIf
    Case #WM_DESTROY
      RemoveProp_(hWnd, #Scroller_WINPROP$)
      FreeMemory(*this)
      result = CallWindowProc_(oldProc, hWnd, uMsg, wParam, lParam)
      SetWindowLongPtr_(hWnd, #GWL_WNDPROC, oldProc)
    Case #WM_HSCROLL, #WM_VSCROLL
      If scrollCode <> #SB_ENDSCROLL
        ;Despatach an appropriate #Scroller_ONSCROLL event.
          event\eventCode = #Scroller_ONSCROLL
          If uMsg = #WM_HSCROLL
            event\blnHorizontalScrollBar = #True
          EndIf
          scrollCode = wParam&$ffff
          Select scrollCode
            Case #SB_LINELEFT
              event\scrollCode = #Scroller_LINELEFTUP
            Case #SB_LINERIGHT
              event\scrollCode = #Scroller_LINERIGHTDOWN
            Case #SB_PAGELEFT
              event\scrollCode = #Scroller_PAGELEFTUP
            Case #SB_PAGERIGHT
              event\scrollCode = #Scroller_PAGERIGHTDOWN
            Case #SB_THUMBTRACK
              event\scrollCode = #Scroller_THUMBTRACK
            Case #SB_THUMBPOSITION
              event\scrollCode = #Scroller_THUMBPOSITION
          EndSelect
        Scroller_DespatchEvent(*this, event)
      EndIf
    Case #WM_MOUSEWHEEL
      If *this\documentHeight > *this\pageHeight
      ;Obtain the wheel increment in units of #WHEEL_DELTA (120).
        zDelta = wParam>>16&$ffff
        zDelta / #WHEEL_DELTA
        If zDelta
          ;Despatach an appropriate #Scroller_ONSCROLL event.
            event\eventCode = #Scroller_ONSCROLL
            event\scrollCode = #Scroller_MOUSEWHEEL
            event\scrollMouseWheelDelta = zDelta
            Scroller_DespatchEvent(*this, event)
        EndIf
      EndIf
    Case #WM_KEYDOWN
      result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
      ;Here we affect a scroll if the user pushed an appropriate key and the #Scroller_ADDKEYBOARDINTERFACE style is set.
      If *this\flags & #Scroller_ADDKEYBOARDCONTROL
        ;Despatach an appropriate #Scroller_ONSCROLL event.
          event\eventCode = #Scroller_ONSCROLL
        Select wParam
          Case #VK_LEFT
            If *this\documentWidth > *this\pageWidth
              event\blnHorizontalScrollBar = #True
              event\scrollCode = #Scroller_LINELEFTUP
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_UP
            If *this\documentHeight > *this\pageHeight
              event\scrollCode = #Scroller_LINELEFTUP
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_RIGHT
            If *this\documentWidth > *this\pageWidth
              event\blnHorizontalScrollBar = #True
              event\scrollCode = #Scroller_LINERIGHTDOWN
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_DOWN
            If *this\documentHeight > *this\pageHeight
              event\scrollCode = #Scroller_LINERIGHTDOWN
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_PRIOR
            If *this\documentHeight > *this\pageHeight
              event\scrollCode = #Scroller_PAGELEFTUP
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_NEXT
            If *this\documentHeight > *this\pageHeight
              event\scrollCode = #Scroller_PAGERIGHTDOWN
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_HOME
            If *this\documentHeight > *this\pageHeight Or *this\documentWidth > *this\pageWidth
              event\scrollCode = #Scroller_TOP
              Scroller_DespatchEvent(*this, event)
            EndIf
          Case #VK_END
            If *this\documentHeight > *this\pageHeight Or *this\documentWidth > *this\pageWidth
              event\scrollCode = #Scroller_BOTTOM
              Scroller_DespatchEvent(*this, event)
            EndIf
        EndSelect
      EndIf
    Default 
      result = CallWindowProc_(oldProc, hWnd, uMsg, wParam, lParam)
  EndSelect
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-CLASS METHODS
;==============

;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_Destroy(*this._Scroller)
  FreeGadget(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_Disable(*this._Scroller, state)
  DisableGadget(*this\conID, state)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetHeight(*this._Scroller)
  ProcedureReturn GadgetHeight(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetID(*this._Scroller)
  ProcedureReturn *this\conID
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetUserData(*this._Scroller)
  ProcedureReturn GetGadgetData(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetWidth(*this._Scroller)
  ProcedureReturn GadgetWidth(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetX(*this._Scroller)
  ProcedureReturn GadgetX(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetY(*this._Scroller)
  ProcedureReturn GadgetY(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_Hide(*this._Scroller, state)
  HideGadget(*this\conID, state)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_Resize(*this._Scroller, x, y, width, height)
  Protected wp.WINDOWPLACEMENT, event.Scroller_EVENT, oldWidth, oldHeight
  ;Retrieve existing dimensions.
    wp\Length = SizeOf(WINDOWPLACEMENT)
    GetWindowPlacement_(*this\conhWnd, wp)
    oldheight = wp\rcNormalPosition\bottom - wp\rcNormalPosition\top
    oldWidth = wp\rcNormalPosition\right - wp\rcNormalPosition\left
  ;Replace any occurrences of #PB_Ignore.
    If x = #PB_Ignore
      x = wp\rcNormalPosition\left
    EndIf
    If y = #PB_Ignore
      y = wp\rcNormalPosition\top
    EndIf
    If width = #PB_Ignore
      width = oldWidth
    EndIf
    If height = #PB_Ignore
      height = oldHeight
    EndIf
  ;Now resize the control.
    ResizeGadget(*this\conID, x, y, width, height)
  ;Reset the page sizes.
    Scroller_SetPageSizes(*this._Scroller)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_SetFocus(*this._Scroller)
  SetActiveGadget(*this\conID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_SetUserData(*this._Scroller, userData)
  SetGadgetData(*this\conID, userData)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetpageHeight(*this._Scroller)
  ProcedureReturn *this\pageHeight
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetpageWidth(*this._Scroller)
  ProcedureReturn *this\pageWidth
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetDocumentHeight(*this._Scroller)
  Protected result
  result = *this\documentHeight
  If result = #Scroller_AUTOSIZE
    result = *this\pageHeight
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_GetDocumentWidth(*this._Scroller)
  Protected result
  result = *this\documentWidth
  If result = #Scroller_AUTOSIZE
    result = *this\pageWidth
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_GetScrollPositions(*this._Scroller, *scrollX.INTEGER, *scrollY.INTEGER)
  If *scrollX
    *scrollX\i = *this\scrollX
  EndIf
  If *scrollY
    *scrollY\i = *this\scrollY
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_ResizedocumentHeight(*this._Scroller, documentHeight=#Scroller_AUTOSIZE)
  ;Only proceed if the new height is truly different to the existing one.
  If documentHeight  <> *this\documentHeight
    If documentHeight < 0
      documentHeight = #Scroller_AUTOSIZE
    EndIf
    *this\documentHeight = documentHeight
    Scroller_SetPageSizes(*this)
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_ResizedocumentWidth(*this._Scroller, documentWidth=#Scroller_AUTOSIZE)
  ;Only proceed if the new width is truly different to the existing one.
  If documentWidth  <> *this\documentWidth
    If documentWidth < 0
      documentWidth = #Scroller_AUTOSIZE
    EndIf
    *this\documentWidth = documentWidth
    Scroller_SetPageSizes(*this)
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_SetAutoScroll(*this._Scroller, scrollStep=0)
  If scrollStep < 0
    scrollStep = 0
  EndIf
  *this\autoScrollStep = scrollStep
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_SetScrollX(*this._Scroller, scrollX, blnInvalidateControl=#True)
  Protected sinf.SCROLLINFO
  ;Prevent unnecessary setting of the scroll values.
    If scrollX > *this\documentWidth - *this\pageWidth
      scrollX = *this\documentWidth - *this\pageWidth
    EndIf
    If scrollX < 0
      scrollX = 0
    EndIf
  ;Only proceed if the new scroll position differs from the existing one.
    If scrollX <> *this\scrollX
      sinf\cbSize = SizeOf(SCROLLINFO)
      sinf\fMask = #SIF_POS
      sinf\nPos = scrollX
      SetScrollInfo_(*this\conhWnd, #SB_HORZ, sinf, #True)
      *this\scrollX = scrollX
      If blnInvalidateControl
        InvalidateRect_(*this\conhWnd, 0, 1)
      EndIf
    EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure Method_SetScrollY(*this._Scroller, scrollY, blnInvalidateControl=#True)
  Protected sinf.SCROLLINFO
  ;Prevent unnecessary setting of the scroll values.
    If scrollY > *this\documentHeight - *this\pageHeight
      scrollY = *this\documentHeight - *this\pageHeight
    EndIf
    If scrollY < 0
      scrollY = 0
    EndIf
  ;Only proceed if the new scroll position differs from the existing one.
    If scrollY <> *this\scrollY
      sinf\cbSize = SizeOf(SCROLLINFO)
      sinf\fMask = #SIF_POS
      sinf\nPos = scrollY
      SetScrollInfo_(*this\conhWnd, #SB_VERT, sinf, #True)
      *this\scrollY = scrollY
      If blnInvalidateControl
        InvalidateRect_(*this\conhWnd, 0, 1)
      EndIf
    EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i Method_SetEventHandler(*this._Scroller, ptrHandler)
  Protected result
  result = *this\eventProc
  *this\eventProc = ptrHandler
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-VIRTUAL TABLES
;===============

;/////////////////////////////////////////////////////////////////////////////////
  DataSection
    Scroller_VT:
      Data.i @Method_Destroy()
      Data.i @Method_Disable()
      Data.i @Method_GetHeight()
      Data.i @Method_GetID()
      Data.i @Method_GetUserData()
      Data.i @Method_GetWidth()
      Data.i @Method_GetX()
      Data.i @Method_GetY()
      Data.i @Method_Hide()
      Data.i @Method_Resize()
      Data.i @Method_SetFocus()
      Data.i @Method_SetUserData()
      Data.i @Method_GetPageHeight()
      Data.i @Method_GetPageWidth()
      Data.i @Method_GetDocumentHeight()
      Data.i @Method_GetDocumentWidth()
      Data.i @Method_GetScrollPositions()
      Data.i @Method_ResizedocumentHeight()
      Data.i @Method_ResizedocumentWidth()
      Data.i @Method_SetAutoScroll()
      Data.i @Method_SetScrollX()
      Data.i @Method_SetScrollY()
      Data.i @Method_SetEventHandler()
  EndDataSection
;/////////////////////////////////////////////////////////////////////////////////

DisableExplicit

EndModule

CompilerEndIf
; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 269
; FirstLine = 249
; Folding = -------
; Markers = 216
; EnableUnicode
; EnableThread
; Executable = nxReportU.dll